

#################################################################################################################
# This is the Monte Carlo Simulation for the paper "Quantile and Probability Curves Without Crossing" by	# 
# V. Chernozhukov, I. Fernandez-Val, and Alfred Galinchon							#
#################################################################################################################


# This version: 10/22/2009

rm(list = ls());

#########################################################################################
# DESIGN CALIBRATED TO ABADIE (2002) VIETNAM VETERANS APPLICATION
#########################################################################################


#########################################################################################
# I - PRELIMINARY FUNCTIONS								#
#########################################################################################


# Remove repeated elements of a sorted vector
remove_repeated <- function(v)
	{;
	v	<- as.vector(v);
	vv	<- v[1];
	last	<- 1;
	for (i in 2:length(v)) {;
        	if (v[i] != vv[last]) {;
			vv 	<- c(vv, v[i]);
			last	<- last+1;
			};
		};
	return(vv);
	};


# Compute IVQR estimator for binary-regressor and binary-instrument (Chernozhukov and Hansen, 2005)
ivqr <- function(y, d, z, taus, gridalpha)
	{;
	y_z0		<- y[z==0];
	y_z1		<- y[z==1];
	d_z0		<- d[z==0];
	d_z1		<- d[z==1];
	yy_z0		<- kronecker(y_z0, matrix(1, ncol = length(gridalpha), nrow = 1)) - kronecker(d_z0, matrix(gridalpha, nrow = 1));
	yy_z1		<- kronecker(y_z1, matrix(1, ncol = length(gridalpha), nrow = 1)) - kronecker(d_z1, matrix(gridalpha, nrow = 1));
	dquantile	<- abs( apply(yy_z1, 2, quantile, taus) - apply(yy_z0, 2, quantile, taus) );
	dquantile	<- apply(dquantile + kronecker(matrix(1, nrow = length(taus), ncol = 1), matrix(abs(gridalpha), nrow = 1))/10000000, 1, rank);
	alpha		<- apply(kronecker(gridalpha, matrix(1, ncol = length(taus), nrow = 1)) * (dquantile == 1), 2, sum);
	yy		<- kronecker(y, matrix(1, ncol = length(alpha), nrow = 1)) - kronecker(d, matrix(alpha, nrow = 1));
	quantile_d0	<- diag(apply(yy, 2, quantile, taus));
        return(cbind(quantile_d0, quantile_d0+alpha)); 
	};


# Compute IVDR estimator of Abadie (2002) (based on equations 5 and 6)
ivdr <- function(y, d, z, ys)
	{;
	n	<- length(y);
	nz1	<- sum(z);
	nz0	<- n - nz1;

	F0	<- NULL;
	F1	<- NULL;

	for (yi in ys) {;
		numF1	<- mean(as.real(y <= yi) * d * z) * (n / nz1) -  mean(as.real(y <= yi) * d * (1 - z)) * (n / nz0);
		denF1	<- mean(d * z) * (n / nz1) -  mean(d * (1 - z)) * (n / nz0);
		F1	<- cbind(F1, numF1/denF1);

		numF0	<- mean(as.real(y <= yi) * (1 - d) * z) * (n / nz1) -  mean(as.real(y <= yi) * (1 - d) * (1 - z)) * (n / nz0);
		denF0	<- mean((1 - d) * z) * (n / nz1) -  mean((1 - d) * (1 - z)) * (n / nz0);
		F0	<- cbind(F0, numF0/denF0);
		};

	return(rbind(F0, F1));
	};




#########################################################################################
# II - MAIN PROGRAM									#
#########################################################################################

library(foreign);


######################################################################################### 
# DATA: ABADIE (2002), JASA 97								#
######################################################################################### 
# VARIABLES:										#
#											#
# 1 - awage: Y		annual wage							#
# 2 - wwage		weekly wage							#
# 3 - veteran: D	veteran status							#
# 4 - draft: Z		draft eligibility						#
######################################################################################### 

data<-read.dta("z:/quantile_curves/IVQR/data/vietnam.dta");
attach(data);
n <- nrow(data);

######################################################################################### 
# CALIBRATING THE DESIGN 								#
######################################################################################### 

#########################################################################################
# LOCATION MODEL
#
# Y = b0 + b1 D + u
# D = 1(a0 + a1 Z + v >= 0)
# (u,v) ~ N2, Z fixed to the data set
# Calibration
#	1 - var(v) = 1
#	2 - a0 = invnorm(P(D=1|Z=0)), a1 = invnorm(P(D=1|Z=1)) - invnorm(P(D=1|Z=0))
#	3 - m = a0 + a1 Z, lambda = phi(m) (D - Phi(m))/[Phi(m)(1 - Phi(m))]
#	4 - OLS of Y on D, lambda -> b0, b1, cov(u,v), var(res)
#	5 - var(u) = var(res) + cov(u,v)^2
#########################################################################################


# REDUCED FORM PARAMETERS;

pi0	<- qnorm(mean(veteran[draft==0]));
pi1	<- qnorm(mean(veteran[draft==1])) - qnorm(mean(veteran[draft==0]));
m	<- pi0 + pi1*draft;
lambda	<- dnorm(m) * (veteran - pnorm(m)) / (pnorm(m) * (1 - pnorm(m)));


# STRUCTURAL FORM PARAMETERS;

fit1	<- lm(awage ~ veteran + lambda);
betas	<- fit1$coef[1:2];
cov	<- fit1$coef[3];
var	<- summary.lm(fit1)$sigma^2;

# QUANTILE FUNCTIONS;

taus		<- c(0:1000)/1000;
Q0		<- betas[1] + qnorm(mean=0, sd = sqrt(var), taus);
Q1		<- betas[1] + betas[2] + qnorm(mean=0, sd = sqrt(var), taus);

fQ0		<- approxfun(taus, Q0);
fQ1		<- approxfun(taus, Q1);
fF0		<- approxfun(Q0, taus);
fF1		<- approxfun(Q1, taus);

nn		<- 1000000;
ZZ		<- rbinom(nn, 1, mean(draft));
vv		<- rnorm(nn);
uu		<- cov * vv + rnorm(nn, sd = sqrt(var));
DD		<- (pi0 + pi1*ZZ + vv > 0);
YY		<- betas[1] + betas[2] * DD + uu;

# TRUE QUANTILE AND DISTRIBUTION FUNCTIONS;

taus.s		<- c(2:98)/100;
taus.s		<- c(4:196)/200;
taus2.s		<- c(2:98)/100;
taus2.s		<- c(4:196)/200;
ys.s		<- remove_repeated(quantile(YY, taus2.s));

Q0.0		<- fQ0(taus.s);
Q1.0		<- fQ1(taus.s);
F0.0		<- fF0(ys.s)
F1.0		<- fF1(ys.s)

z		<- draft;

######################################################################################### 
# SIMULATION  		 								#
######################################################################################### 

# PARAMETERS OF THE SIMULATION
n		<- length(awage); 	# sample size
B		<- 5000; 		# Number of replications
set.seed(88);
gridalpha	<- 5*c(-2000:300);


Q0_mc	<- NULL;
Q1_mc	<- NULL;
rQ0_mc	<- NULL;
rQ1_mc	<- NULL;
iQ0_mc	<- NULL;
iQ1_mc	<- NULL;

F0_mc	<- NULL;
F1_mc	<- NULL;
rF0_mc	<- NULL;
rF1_mc	<- NULL;
iF0_mc	<- NULL;
iF1_mc	<- NULL;



for (s in 1:B) {;

# Generating the data

	v	<- rnorm(n);
	u	<- cov * v + rnorm(n, sd = sqrt(var));
	d	<- (pi0 + pi1*z + v > 0);
	y	<- betas[1] + betas[2] * d + u;
	

# Estimate distribution functions;

	idr	<- ivdr(y, d, z, ys.s);
	F0	<- idr[1, ];
	F1	<- idr[2, ];

	rF0	<- sort(F0);
	rF1	<- sort(F1);


	iF0	<- isoreg(ys.s, F0)$yf;
	iF1	<- isoreg(ys.s, F1)$yf;



	F0_mc	<-  cbind(F0_mc, F0 * ((F0 >= 0) & (F0 <= 1)) + 1 * (F0 > 1) ); 
	F1_mc	<-  cbind(F1_mc, F1 * ((F1 >= 0) & (F1 <= 1)) + 1 * (F1 > 1) ); 
	rF0_mc	<-  cbind(rF0_mc, rF0 * ((rF0 >= 0) & (rF0 <= 1)) + 1 * (rF0 > 1) ); 
	rF1_mc	<-  cbind(rF1_mc, rF1 * ((rF1 >= 0) & (rF1 <= 1)) + 1 * (rF1 > 1) ); 
	iF0_mc	<-  cbind(iF0_mc, iF0 * ((iF0 >= 0) & (iF0 <= 1)) + 1 * (iF0 > 1) ); 
	iF1_mc	<-  cbind(iF1_mc, iF1 * ((iF1 >= 0) & (iF1 <= 1)) + 1 * (iF1 > 1) ); 


# Estimate quantile functions;

	iqr	<- ivqr(y, d, z, taus.s, gridalpha);

	Q0	<- iqr[ ,1];
	Q1	<- iqr[ ,2];

	Q0_mc	<- cbind(Q0_mc, Q0);
	Q1_mc	<- cbind(Q1_mc, Q1);
	rQ0_mc	<- cbind(rQ0_mc, sort(Q0) );
	rQ1_mc	<- cbind(rQ1_mc, sort(Q1) );
	iQ0_mc	<- cbind(iQ0_mc, isoreg(taus.s, Q0)$yf );
	iQ1_mc	<- cbind(iQ1_mc, isoreg(taus.s, Q1)$yf );


	};





# APPROXIMATION PROPERTIES;

u_mc 		 <- kronecker(matrix(rep(1, B), nrow=1), matrix(taus.s,ncol=1));
ulead_mc	 <- rbind(u_mc[2:nrow(u_mc), ], matrix(1, ncol = B, nrow = 1));
y_mc 		 <- kronecker(matrix(rep(1, B), nrow=1), matrix(ys.s,ncol=1));
ylead_mc	 <- rbind(y_mc[2:nrow(y_mc), ], matrix(max(ys.s), ncol = B, nrow = 1));

error_Q0	<- abs(Q0_mc - kronecker(Q0.0, matrix(1, ncol = B, nrow = 1)));
error_rQ0	<- abs(rQ0_mc - kronecker(Q0.0, matrix(1, ncol = B, nrow = 1)));
error_iQ0	<- abs(iQ0_mc - kronecker(Q0.0, matrix(1, ncol = B, nrow = 1)));
error_Q1	<- abs(Q1_mc - kronecker(Q1.0, matrix(1, ncol = B, nrow = 1)));
error_rQ1	<- abs(rQ1_mc - kronecker(Q1.0, matrix(1, ncol = B, nrow = 1)));
error_iQ1	<- abs(iQ1_mc - kronecker(Q1.0, matrix(1, ncol = B, nrow = 1)));


error_F0	<- abs(F0_mc - kronecker(F0.0, matrix(1, ncol = B, nrow = 1)));
error_rF0	<- abs(rF0_mc - kronecker(F0.0, matrix(1, ncol = B, nrow = 1)));
error_iF0	<- abs(iF0_mc - kronecker(F0.0, matrix(1, ncol = B, nrow = 1)));
error_F1	<- abs(F1_mc - kronecker(F1.0, matrix(1, ncol = B, nrow = 1)));
error_rF1	<- abs(rF1_mc - kronecker(F1.0, matrix(1, ncol = B, nrow = 1)));
error_iF1	<- abs(iF1_mc - kronecker(F1.0, matrix(1, ncol = B, nrow = 1)));


error_QE	<- abs(Q1_mc - Q0_mc - kronecker(Q1.0 - Q0.0, matrix(1, ncol = B, nrow = 1)));
error_rQE	<- abs(rQ1_mc - rQ0_mc - kronecker(Q1.0 - Q0.0, matrix(1, ncol = B, nrow = 1)));
error_iQE	<- abs(iQ1_mc - iQ0_mc - kronecker(Q1.0 - Q0.0, matrix(1, ncol = B, nrow = 1)));


error_FE	<- abs(F1_mc - F0_mc - kronecker(F1.0 - F0.0, matrix(1, ncol = B, nrow = 1)));
error_rFE	<- abs(rF1_mc - rF0_mc - kronecker(F1.0 - F0.0, matrix(1, ncol = B, nrow = 1)));
error_iFE	<- abs(iF1_mc - iF0_mc - kronecker(F1.0 - F0.0, matrix(1, ncol = B, nrow = 1)));



table <- matrix(0, nrow = 3, ncol = 12, dimnames = list(c("L1", "L2", "Linf"), c("Q0", "rQ0/Q0", "iQ0/Q0", "Q1", "rQ1/Q1", "iQ1/Q1", "F0", "rF0/F0", "iF0/F0", "F1", "rF1/F1", "iF1/F1")));

table[1,1]	<- mean(sum((ulead_mc - u_mc) / sum(ulead_mc - u_mc) * error_Q0)); 
table[1,2]	<- mean(sum((ulead_mc - u_mc) / sum(ulead_mc - u_mc) * error_rQ0)) / table[1,1]; 
table[1,3]	<- mean(sum((ulead_mc - u_mc) / sum(ulead_mc - u_mc) * error_iQ0)) / table[1,1]; 
table[1,4]	<- mean(sum((ulead_mc - u_mc) / sum(ulead_mc - u_mc) * error_Q1)); 
table[1,5]	<- mean(sum((ulead_mc - u_mc) / sum(ulead_mc - u_mc) * error_rQ1)) / table[1,4]; 
table[1,6]	<- mean(sum((ulead_mc - u_mc) / sum(ulead_mc - u_mc) * error_iQ1)) / table[1,4]; 
table[1,7]	<- mean(sum((ylead_mc - y_mc) / sum(ylead_mc - y_mc) * error_F0)); 
table[1,8]	<- mean(sum((ylead_mc - y_mc) / sum(ylead_mc - y_mc) * error_rF0)) / table[1,7]; 
table[1,9]	<- mean(sum((ylead_mc - y_mc) / sum(ylead_mc - y_mc) * error_iF0)) / table[1,7]; 
table[1,10]	<- mean(sum((ylead_mc - y_mc) / sum(ylead_mc - y_mc) * error_F1)); 
table[1,11]	<- mean(sum((ylead_mc - y_mc) / sum(ylead_mc - y_mc) * error_rF1)) / table[1,10]; 
table[1,12]	<- mean(sum((ylead_mc - y_mc) / sum(ylead_mc - y_mc) * error_iF1)) / table[1,10]; 


table[2,1]	<- mean(sum((ulead_mc - u_mc) / sum(ulead_mc - u_mc) * error_Q0^2)^(1/2)); 
table[2,2]	<- mean(sum((ulead_mc - u_mc) / sum(ulead_mc - u_mc) * error_rQ0^2)^(1/2)) / table[2,1]; 
table[2,3]	<- mean(sum((ulead_mc - u_mc) / sum(ulead_mc - u_mc) * error_iQ0^2)^(1/2)) / table[2,1]; 
table[2,4]	<- mean(sum((ulead_mc - u_mc) / sum(ulead_mc - u_mc) * error_Q1^2)^(1/2)); 
table[2,5]	<- mean(sum((ulead_mc - u_mc) / sum(ulead_mc - u_mc) * error_rQ1^2)^(1/2)) / table[2,4]; 
table[2,6]	<- mean(sum((ulead_mc - u_mc) / sum(ulead_mc - u_mc) * error_iQ1^2)^(1/2)) / table[2,4]; 
table[2,7]	<- mean(sum((ylead_mc - y_mc) / sum(ylead_mc - y_mc) * error_F0^2)^(1/2)); 
table[2,8]	<- mean(sum((ylead_mc - y_mc) / sum(ylead_mc - y_mc) * error_rF0^2)^(1/2)) / table[2,7]; 
table[2,9]	<- mean(sum((ylead_mc - y_mc) / sum(ylead_mc - y_mc) * error_iF0^2)^(1/2)) / table[2,7]; 
table[2,10]	<- mean(sum((ylead_mc - y_mc) / sum(ylead_mc - y_mc) * error_F1^2)^(1/2)); 
table[2,11]	<- mean(sum((ylead_mc - y_mc) / sum(ylead_mc - y_mc) * error_rF1^2)^(1/2)) / table[2,10]; 
table[2,12]	<- mean(sum((ylead_mc - y_mc) / sum(ylead_mc - y_mc) * error_iF1^2)^(1/2)) / table[2,10]; 


table[3,1]	<- mean(apply(error_Q0, 2, max));
table[3,2]	<- mean(apply(error_rQ0, 2, max)) / table[3,1];
table[3,3]	<- mean(apply(error_iQ0, 2, max)) / table[3,1];
table[3,4]	<- mean(apply(error_Q1, 2, max));
table[3,5]	<- mean(apply(error_rQ1, 2, max)) / table[3,4];
table[3,6]	<- mean(apply(error_iQ1, 2, max)) / table[3,4];
table[3,7]	<- mean(apply(error_F0, 2, max));
table[3,8]	<- mean(apply(error_rF0, 2, max)) / table[3,7];
table[3,9]	<- mean(apply(error_iF0, 2, max)) / table[3,7];
table[3,10]	<- mean(apply(error_F1, 2, max));
table[3,11]	<- mean(apply(error_rF1, 2, max)) / table[3,10];
table[3,12]	<- mean(apply(error_iF1, 2, max)) / table[3,10];

table

table <- matrix(0, nrow = 3, ncol = 6, dimnames = list(c("L1", "L2", "Linf"), c("QE", "rQE/QE", "iQE/QE", "FE", "rFE/FE", "iFE/FE")));

table[1,1]	<- mean(sum((ulead_mc - u_mc) / sum(ulead_mc - u_mc) * error_QE)); 
table[1,2]	<- mean(sum((ulead_mc - u_mc) / sum(ulead_mc - u_mc) * error_rQE)) / table[1,1]; 
table[1,3]	<- mean(sum((ulead_mc - u_mc) / sum(ulead_mc - u_mc) * error_iQE)) / table[1,1]; 
table[1,4]	<- mean(sum((ylead_mc - y_mc) / sum(ylead_mc - y_mc) * error_FE)); 
table[1,5]	<- mean(sum((ylead_mc - y_mc) / sum(ylead_mc - y_mc) * error_rFE)) / table[1,4]; 
table[1,6]	<- mean(sum((ylead_mc - y_mc) / sum(ylead_mc - y_mc) * error_iFE)) / table[1,4]; 

table[2,1]	<- mean(sum((ulead_mc - u_mc) / sum(ulead_mc - u_mc) * error_QE^2)^(1/2)); 
table[2,2]	<- mean(sum((ulead_mc - u_mc) / sum(ulead_mc - u_mc) * error_rQE^2)^(1/2)) / table[2,1]; 
table[2,3]	<- mean(sum((ulead_mc - u_mc) / sum(ulead_mc - u_mc) * error_iQE^2)^(1/2)) / table[2,1]; 
table[2,4]	<- mean(sum((ylead_mc - y_mc) / sum(ylead_mc - y_mc) * error_FE^2)^(1/2)); 
table[2,5]	<- mean(sum((ylead_mc - y_mc) / sum(ylead_mc - y_mc) * error_rFE^2)^(1/2)) / table[2,4]; 
table[2,6]	<- mean(sum((ylead_mc - y_mc) / sum(ylead_mc - y_mc) * error_iFE^2)^(1/2)) / table[2,4]; 


table[3,1]	<- mean(apply(error_QE, 2, max));
table[3,2]	<- mean(apply(error_rQE, 2, max)) / table[3,1];
table[3,3]	<- mean(apply(error_iQE, 2, max)) / table[3,1];
table[3,4]	<- mean(apply(error_FE, 2, max));
table[3,5]	<- mean(apply(error_rFE, 2, max)) / table[3,4];
table[3,6]	<- mean(apply(error_iFE, 2, max)) / table[3,4];

table


detach(data);
rm();


ls();
q();