

#########################################################################################################
# This is the empirical example for the paper "Quantile and Probability Curves Without Crossing" by	# 
# V. Chernozhukov, I. Fernandez-Val, and Alfred Galinchon						#
#########################################################################################################

#################################################################################
# PROGRAM: POINT ESTIMATES OF STRUCTURAL DISTRIBUTION AND QUANTILE FUNCTIONS	#
# (USING DIRECT PROCEDURE FOR REARRANGEMENT)		  			#
#################################################################################

# This version: 10/22/2009

rm(list = ls());

#########################################################################################
# I - PRELIMINARY FUNCTIONS								#
#########################################################################################


# Remove repeated elements of a sorted vector
remove_repeated <- function(v)
	{;
	v	<- as.vector(v);
	vv	<- v[1];
	last	<- 1;
	for (i in 2:length(v)) {;
        	if (v[i] != vv[last]) {;
			vv 	<- c(vv, v[i]);
			last	<- last+1;
			};
		};
	return(vv);
	};



# Compute IVQR estimator for binary-regressor and binary-instrument (Chernozhukov and Hansen, 2005)
ivqr <- function(y, d, z, taus, gridalpha)
	{;
	y_z0		<- y[z==0];
	y_z1		<- y[z==1];
	d_z0		<- d[z==0];
	d_z1		<- d[z==1];
	yy_z0		<- kronecker(y_z0, matrix(1, ncol = length(gridalpha), nrow = 1)) - kronecker(d_z0, matrix(gridalpha, nrow = 1));
	yy_z1		<- kronecker(y_z1, matrix(1, ncol = length(gridalpha), nrow = 1)) - kronecker(d_z1, matrix(gridalpha, nrow = 1));
	dquantile	<- abs( apply(yy_z1, 2, quantile, taus) - apply(yy_z0, 2, quantile, taus) );
	dquantile	<- apply(dquantile + kronecker(matrix(1, nrow = length(taus), ncol = 1), matrix(abs(gridalpha), nrow = 1))/10000000, 1, rank);
	alpha		<- apply(kronecker(gridalpha, matrix(1, ncol = length(taus), nrow = 1)) * (dquantile == 1), 2, sum);
	yy		<- kronecker(y, matrix(1, ncol = length(alpha), nrow = 1)) - kronecker(d, matrix(alpha, nrow = 1));
	quantile_d0	<- diag(apply(yy, 2, quantile, taus));
        return(cbind(quantile_d0, quantile_d0+alpha)); 
	};



# Compute IVDR estimator of Abadie (2002) (based on equations 5 and 6)
ivdr <- function(y, d, z, ys)
	{;
	n	<- length(y);
	nz1	<- sum(z);
	nz0	<- n - nz1;

	F0	<- NULL;
	F1	<- NULL;

	for (yi in ys) {;
		numF1	<- mean(as.real(y <= yi) * d * z) * (n / nz1) -  mean(as.real(y <= yi) * d * (1 - z)) * (n / nz0);
		denF1	<- mean(d * z) * (n / nz1) -  mean(d * (1 - z)) * (n / nz0);
		F1	<- cbind(F1, numF1/denF1);

		numF0	<- mean(as.real(y <= yi) * (1 - d) * z) * (n / nz1) -  mean(as.real(y <= yi) * (1 - d) * (1 - z)) * (n / nz0);
		denF0	<- mean((1 - d) * z) * (n / nz1) -  mean((1 - d) * (1 - z)) * (n / nz0);
		F0	<- cbind(F0, numF0/denF0);
		};

	return(rbind(F0, F1));
	};


#########################################################################################
# II - MAIN PROGRAM									#
#########################################################################################

library(foreign);
library(quantreg);
library(matlab);

######################################################################################### 
# DATA: ABADIE (2002), JASA 97								#
######################################################################################### 
# VARIABLES:										#
#											#
# 1 - awage	annual wage								#
# 2 - wwage	weekly wage								#
# 3 - veteran   veteran status								#
# 4 - draft	draft eligibility							#
######################################################################################### 

data<-read.dta("z:/quantile_curves/IVQR/data/vietnam.dta");
attach(data);
n <- nrow(data);


######################################################################################### 
# DISTRIBUTION FUNCTIONS FOR COMPLIERS (FROM EQ 5 AND 6 OF ABADIE, 2002)		#
######################################################################################### 

ys	<- remove_repeated(sort(awage));
us	<- (ys - min(ys)) / (max(ys) - min(ys));

idr	<- ivdr(awage, veteran, draft, ys);
F0	<- idr[1, ];
F1	<- idr[2, ];

# REARRANGEMENT;
rF0	<- sort(F0);
rF1	<- sort(F1);


# TRIMMING CDF'S TO BE BETWEEN 0 AND 1;
tF0	<-  F0 * ((F0 >= 0) & (F0 <= 1)) + 1 * (F0 > 1); 
tF1	<-  F1 * ((F1 >= 0) & (F1 <= 1)) + 1 * (F1 > 1); 
trF0	<-  rF0 * ((rF0 >= 0) & (rF0 <= 1)) + 1 * (rF0 > 1); 
trF1	<-  rF1 * ((rF1 >= 0) & (rF1 <= 1)) + 1 * (rF1 > 1); 




######################################################################################### 
# QUANTILE FUNCTIONS (IVQR ESTIMATOR OF CHERNOZHUKOV AND HANSEN, 2006)			#
######################################################################################### 


taus		<- c(0:1000)/1000;
gridalpha	<- 5*c(-3000:1000);
iqr		<- ivqr(awage, veteran, draft, taus, gridalpha);

Q0		<- iqr[ ,1];
Q1		<- iqr[ ,2];

# REARRANGEMENT;
rQ0		<- sort(Q0);
rQ1		<- sort(Q1);




######################################################################################### 
# RESULTS										#
######################################################################################### 


# FIGURE 3: ESTIMATES OF STRUCTURAL DISTRIBUTION FUNCTIONS;

postscript("z:/Quantile_Curves/IVQR/results/estimates_distribution.eps",horizontal=FALSE,onefile=FALSE,height=4.9,width=8.4,pointsize=10);

par(mfrow=c(1,2));


plot(c(0,50000),range(tF0), type="n", xlab="Annual Earnings", ylab="Probability", main="");
title(main= "A - Original curves");
par(lty = 1, col = 1, lwd = 1);
lines(ys, tF0);
par(lty = 1, lwd = 1, col = 2);
lines(ys, tF1);
par(lty = 1, lwd = 1, col = 1);
legend(25000, .2, c("Non-veterans", "Veterans"), lty = c(1, 1), lwd = c(1, 1), col = c(1,2), bty = "n");

plot(c(0,50000),range(tF0), type="n", xlab="Annual Earnings", ylab="Probability", main="");
title(main= "B - Rearranged curves");
par(lty = 1, col = 1, lwd = 1);
lines(ys, trF0);
par(lty = 1, lwd = 1, col = 2);
lines(ys, trF1);
par(lty = 1, lwd = 1, col = 1);
legend(25000, .2, c("Non-veterans", "Veterans"), lty = c(1, 1), lwd = c(1, 1), col = c(1,2), bty = "n");



dev.off();






# FIGURE 4: ESTIMATES OF STRUCTURAL QUANTILE FUNCTIONS;

range <- (taus>=.1 & taus<=.9);


postscript("z:/Quantile_Curves/IVQR/results/estimates_quantile.eps",horizontal=FALSE,onefile=FALSE,height=4.9,width=8.4,pointsize=10);

par(mfrow=c(1,2));


plot(range(taus[range]),range(Q0[range]), type="n", ylab="Annual earnings", xlab="Quantile index", main="");
title(main= "A - Original curves");
par(lty = 1, col = 1, lwd = 1);
lines(taus[range], Q0[range]);
par(lty = 1, lwd = 1, col = 2);
lines(taus[range], Q1[range]);
par(lty = 1, lwd = 1, col = 1);
legend(.1, 20000, c("Non-veterans", "Veterans"), lty = c(1, 1), lwd = c(1, 1), col = c(1,2), bty = "n");

plot(range(taus[range]),range(Q0[range]), type="n", ylab="Annual earnings", xlab="Quantile index", main="");
title(main= "B - Rearranged curves");
par(lty = 1, col = 1, lwd = 1);
lines(taus[range], rQ0[range]);
par(lty = 1, lwd = 1, col = 2);
lines(taus[range], rQ1[range]);
par(lty = 1, lwd = 1, col = 1);
legend(.1, 20000, c("Non-veterans", "Veterans"), lty = c(1, 1), lwd = c(1, 1), col = c(1,2), bty = "n");



dev.off();






ls();
q();