
#########################################################################################################
# This is the empirical example for the paper "Quantile and Probability Curves Without Crossing" by	# 
# V. Chernozhukov, I. Fernandez-Val, and Alfred Galinchon						#
#########################################################################################################

###############################################################################################
# PROGRAM: UNIFORM BANDS FOR STRUCTURAL QUANTILE FUNCTIONS				      #
# (weighting matrices constructed using bootstrap variances)    			      #
# (uses sorting procedure to obtain rearrangements)					      #
###############################################################################################


# This version: 10/22/2009

rm(list = ls());

#########################################################################################
# I - PRELIMINARY FUNCTIONS								#
#########################################################################################



# Compute IVQR estimator for binary-regressor and binary-instrument (Chernozhukov and Hansen, 2005)
ivqr <- function(y, d, z, taus, gridalpha)
	{;
	y_z0		<- y[z==0];
	y_z1		<- y[z==1];
	d_z0		<- d[z==0];
	d_z1		<- d[z==1];
	yy_z0		<- kronecker(y_z0, matrix(1, ncol = length(gridalpha), nrow = 1)) - kronecker(d_z0, matrix(gridalpha, nrow = 1));
	yy_z1		<- kronecker(y_z1, matrix(1, ncol = length(gridalpha), nrow = 1)) - kronecker(d_z1, matrix(gridalpha, nrow = 1));
	dquantile	<- abs( apply(yy_z1, 2, quantile, taus) - apply(yy_z0, 2, quantile, taus) );
	dquantile	<- apply(dquantile + kronecker(matrix(1, nrow = length(taus), ncol = 1), matrix(abs(gridalpha), nrow = 1))/10000000, 1, rank);
	alpha		<- apply(kronecker(gridalpha, matrix(1, ncol = length(taus), nrow = 1)) * (dquantile == 1), 2, sum);
	yy		<- kronecker(y, matrix(1, ncol = length(alpha), nrow = 1)) - kronecker(d, matrix(alpha, nrow = 1));
	quantile_d0	<- diag(apply(yy, 2, quantile, taus));
        return(cbind(quantile_d0, quantile_d0+alpha)); 
	};



############## obtains the distribution of K by subsampling ##############################################

subsamplek <- function(data=data, n=n, b=b, B=B, formula, R=R, taus, gridalpha)
    {
    cqf0.bootstrap 	<- NULL;
    rcqf0.bootstrap 	<- NULL;
    scqf0.bootstrap 	<- NULL;
    srcqf0.bootstrap 	<- NULL;
    cqf1.bootstrap 	<- NULL;
    rcqf1.bootstrap 	<- NULL;
    scqf1.bootstrap 	<- NULL;
    srcqf1.bootstrap 	<- NULL;
    bootstrap 		<- NULL;
    for (s in 1:B)
        {
	sdata			<- data[sample(n,b,replace=T), ];
	iqr			<- ivqr(sdata$awage, sdata$veteran, sdata$draft, taus, gridalpha);
	cqf0.bootstrap 		<- cbind(cqf0.bootstrap, iqr[ ,1]);
        scqf0.bootstrap		<- cbind(scqf0.bootstrap, ksmooth(taus, iqr[ ,1], kernel = "box", bandwidth = .10, x.points = taus)$y );
	rcqf0.bootstrap  	<- cbind(rcqf0.bootstrap, sort(iqr[ ,1]));
        srcqf0.bootstrap 	<- cbind(srcqf0.bootstrap, ksmooth(taus, sort(iqr[ ,1]), kernel = "box", bandwidth = .10, x.points = taus)$y );
	cqf1.bootstrap 		<- cbind(cqf1.bootstrap, iqr[ ,2]);
        scqf1.bootstrap		<- cbind(scqf1.bootstrap, ksmooth(taus, iqr[ ,2], kernel = "box", bandwidth = .10, x.points = taus)$y );
	rcqf1.bootstrap  	<- cbind(rcqf1.bootstrap, sort(iqr[ ,2]));
        srcqf1.bootstrap 	<- cbind(srcqf1.bootstrap, ksmooth(taus, sort(iqr[ ,2]), kernel = "box", bandwidth = .10, x.points = taus)$y );
        }
    bootstrap$cqf0  	<- cqf0.bootstrap;
    bootstrap$rcqf0  	<- rcqf0.bootstrap;
    bootstrap$scqf0  	<- scqf0.bootstrap;
    bootstrap$srcqf0 	<- srcqf0.bootstrap;
    bootstrap$cqf1   	<- cqf1.bootstrap;
    bootstrap$rcqf1  	<- rcqf1.bootstrap;
    bootstrap$scqf1  	<- scqf1.bootstrap;
    bootstrap$srcqf1 	<- srcqf1.bootstrap;
    return(bootstrap);
    }


########### Compute lower minorant of a function ##############################

minorant <- function(y)
	{;
	minor	<- NULL;
	n	<- length(y);
	minor	<- y[n];
	for (i in 1:(n-1)) {;
		minor	<- c(minor, min(minor[i], y[n-i]));
		};
	return(rev(minor));
	};

########### Compute upper majorant of a function ##############################

majorant <- function(y)
	{;
	major	<- NULL;
	major	<- y[1];
	n	<- length(y);
	for (i in 2:n) {;
		major	<- c(major, max(major[i-1], y[i]));
		};
	return(major);
	};



#########################################################################################
# II - MAIN PROGRAM									#
#########################################################################################

library(foreign);
library(quantreg);
library(matlab);

######################################################################################### 
# DATA: ABADIE (2002), JASA 97								#
######################################################################################### 
# VARIABLES:										#
#											#
# 1 - awage	anuual wage								#
# 2 - wwage	weekly wage								#
# 3 - veteran   veteran status								#
# 4 - draft	draft eligibility							#
######################################################################################### 

data<-read.dta("z:/quantile_curves/IVQR/data/vietnam.dta");
attach(data);
n <- nrow(data);


######################################################################################### 
# UNIFORM BANDS										#
######################################################################################### 

# PARAMETERS;

set.seed(8);	
B 		<- 500;
b 		<- n;
alpha		<- 0.10;
gridalpha	<- 5*c(-2000:300);

taus 		<- c(1:99)/100;
first    	<- sum(as.real(taus <= .10));
last     	<- sum(as.real(taus <= .90));
rang     	<- c(first:last);
taus_fig 	<- taus[rang];




# MODEL ESTIMATION;

iqr		<- ivqr(awage, veteran, draft, taus, gridalpha);
cqf0 		<- iqr[ ,1];
rcqf0  		<- sort(cqf0);
cqf1 		<- iqr[ ,2];
rcqf1  		<- sort(cqf1);
eqf		<- cqf1 - cqf0;
reqf		<- rcqf1 - rcqf0;



# BOOTSTRAP;

bootstrap 	<- subsamplek(data=data, n=n, b=b, B=B, formula, R=R, taus, gridalpha);


cqf0.b		<- bootstrap$cqf0;
rcqf0.b		<- bootstrap$rcqf0;
cqf1.b		<- bootstrap$cqf1;
rcqf1.b		<- bootstrap$rcqf1;
eqf.b		<- cqf1.b - cqf0.b;
reqf.b		<- rcqf1.b - rcqf0.b;

V.cqf0   	<- apply(cqf0.b, 1, var);
V.rcqf0   	<- apply(rcqf0.b, 1, var);
V.cqf1   	<- apply(cqf1.b, 1, var);
V.rcqf1   	<- apply(rcqf1.b, 1, var);
V.eqf   	<- apply(eqf.b, 1, var);
V.reqf   	<- apply(reqf.b, 1, var);

K.cqf0 		<- sqrt((cqf0.b - matrix(cqf0, length(taus), B, byrow = F))^2 / matrix(V.cqf0, length(taus), B, byrow = F));
K.rcqf0		<- sqrt((rcqf0.b - matrix(rcqf0, length(taus), B, byrow = F))^2 / matrix(V.rcqf0, length(taus), B, byrow = F));
K.cqf1 		<- sqrt((cqf1.b - matrix(cqf1, length(taus), B, byrow = F))^2 / matrix(V.cqf1, length(taus), B, byrow = F));
K.rcqf1		<- sqrt((rcqf1.b - matrix(rcqf1, length(taus), B, byrow = F))^2 / matrix(V.rcqf1, length(taus), B, byrow = F));
K.eqf 		<- sqrt((eqf.b - matrix(eqf, length(taus), B, byrow = F))^2 / matrix(V.eqf, length(taus), B, byrow = F));
K.reqf		<- sqrt((reqf.b - matrix(reqf, length(taus), B, byrow = F))^2 / matrix(V.reqf, length(taus), B, byrow = F));

Kmax.cqf0 	<- apply(K.cqf0[rang, ],2,FUN=max, na.rm = T);
Kmax.rcqf0 	<- apply(K.rcqf0[rang, ],2,FUN=max, na.rm = T);
Kmax.cqf1 	<- apply(K.cqf1[rang, ],2,FUN=max, na.rm = T);
Kmax.rcqf1 	<- apply(K.rcqf1[rang, ],2,FUN=max, na.rm = T);
Kmax.eqf 	<- apply(K.eqf[rang, ],2,FUN=max, na.rm = T);
Kmax.reqf 	<- apply(K.reqf[rang, ],2,FUN=max, na.rm = T);

Kalpha.cqf0 	<- quantile(Kmax.cqf0, 1-alpha);
Kalpha.rcqf0 	<- quantile(Kmax.rcqf0, 1-alpha);
Kalpha.cqf1 	<- quantile(Kmax.cqf1, 1-alpha);
Kalpha.rcqf1 	<- quantile(Kmax.rcqf1, 1-alpha);
Kalpha.eqf 	<- quantile(Kmax.eqf, 1-alpha);
Kalpha.reqf 	<- quantile(Kmax.reqf, 1-alpha);


cqf0.se   	<- sqrt(V.cqf0);
rcqf0.se  	<- sqrt(V.rcqf0);
cqf1.se   	<- sqrt(V.cqf1);
rcqf1.se  	<- sqrt(V.rcqf1);
eqf.se   	<- sqrt(V.eqf);
reqf.se  	<- sqrt(V.reqf);




# FIGURE 5: UNIFORM BANDS FOR STRUCTURAL QUANTILE FUNCTIONS;

first    <- sum(as.real(taus <= 0));
last     <- sum(as.real(taus <= 1));
rang     <- c(first:last);


postscript("z:/Quantile_Curves/IVQR/results/cband_all.eps",horizontal=TRUE,onefile=FALSE,height=4.5,width=7.5,pointsize=10);


u.eqf     <- eqf	+ Kalpha.eqf  * eqf.se;
l.eqf     <- eqf 	- Kalpha.eqf  * eqf.se;
u.reqf    <- reqf 	+ Kalpha.reqf * reqf.se;
l.reqf    <- reqf 	- Kalpha.reqf * reqf.se;

u.cqf0     <- cqf0	+ Kalpha.cqf0  * cqf0.se;
l.cqf0     <- cqf0 	- Kalpha.cqf0  * cqf0.se;
u.rcqf0    <- rcqf0 	+ Kalpha.rcqf0 * rcqf0.se;
l.rcqf0    <- rcqf0 	- Kalpha.rcqf0 * rcqf0.se;
u.icqf0    <- minorant(u.cqf0);
l.icqf0    <- majorant(l.cqf0);
u.ircqf0   <- minorant(u.rcqf0);
l.ircqf0   <- majorant(l.rcqf0);

u.cqf1     <- cqf1	+ Kalpha.cqf1  * cqf1.se;
l.cqf1     <- cqf1 	- Kalpha.cqf1  * cqf1.se;
u.rcqf1    <- rcqf1 	+ Kalpha.rcqf1 * rcqf1.se;
l.rcqf1    <- rcqf1 	- Kalpha.rcqf1 * rcqf1.se;
u.icqf1    <- minorant(u.cqf1);
l.icqf1    <- majorant(l.cqf1);
u.ircqf1   <- minorant(u.rcqf1);
l.ircqf1   <- majorant(l.rcqf1);



par(mfrow=c(1,3));

plot(c(0,1),range(c(min(l.icqf1[rang]),max(u.icqf1[rang]))), xlim =c(0, 1), type="n", xlab="u", ylab="Annual earnings", main="");
title(main= "A. Non-Veterans");
lines(taus[rang], u.ircqf0[rang], col= 2, lty = 1, lwd = 1);
lines(taus[rang], l.ircqf0[rang], col= 2, lty = 1, lwd = 1);
lines(taus[rang], u.icqf0[rang], lty = 2, lwd = 1);
lines(taus[rang], l.icqf0[rang], lty = 2, lwd = 1);
par(col=1);
lines(taus[rang],rcqf0[rang], col= 2, lty = 1);
lines(taus[rang],cqf0[rang], lwd = 1, lty = 2);
legend(0, max(u.icqf1[rang]), c("Original", "Rearranged"),lty=c(2,1),bty="n", lwd = c(1,1), col = c(1,2));


plot(c(0,1),range(c(min(l.icqf1[rang]),max(u.icqf1[rang]))), xlim =c(0, 1), type="n", xlab="u", ylab="Annual earnings", main="");
title(main= "B. Veterans");
lines(taus[rang], u.ircqf1[rang], col= 2, lty = 1, lwd = 1);
lines(taus[rang], l.ircqf1[rang], col= 2, lty = 1, lwd = 1);
lines(taus[rang], u.icqf1[rang], lty = 2, lwd = 1);
lines(taus[rang], l.icqf1[rang], lty = 2, lwd = 1);
lines(taus[rang],rcqf1[rang], col=2, lty = 1);
lines(taus[rang],cqf1[rang], lwd = 1, lty = 2);
legend(0, max(u.icqf1[rang]), c("Original", "Rearranged"),lty=c(2,1),bty="n", lwd = c(1,1), col = c(1,2));


plot(c(0,1),range(c(min(l.eqf[rang]),max(u.eqf[rang]))), xlim =c(0, 1), type="n", xlab="u", ylab="Difference in annual earnings", main="");
title(main= "C. Quantile effect");
lines(taus[rang], u.reqf[rang], col= 2, lty = 1, lwd = 1);
lines(taus[rang], l.reqf[rang], col= 2, lty = 1, lwd = 1);
lines(taus[rang], u.eqf[rang], lty = 2, lwd = 1);
lines(taus[rang], l.eqf[rang], lty = 2, lwd = 1);
lines(taus[rang], reqf[rang], col=2, lty = 1);
lines(taus[rang], eqf[rang], lwd = 1, lty = 2);
legend(0, max(u.eqf[rang]), c("Original", "Rearranged"),lty=c(2,1),bty="n", lwd = c(1,1), col = c(1,2));



dev.off();

